@@ -3,7 +3,7 @@ from django.core import serializers
 from django.db import models
 from django.test import TestCase
 from django.utils.datastructures import SortedDict
-from serializers import Serializer, ModelSerializer, DumpDataSerializer
+from serializers import ObjectSerializer, ModelSerializer, DumpDataSerializer
 from serializers.fields import Field, NaturalKeyRelatedField
 
 
@@ -35,13 +35,13 @@ class TestBasicObjects(SerializationTestCase):
     def test_list(self):
         obj = []
         expected = '[]'
-        output = Serializer().encode(obj, 'json')
+        output = ObjectSerializer().serialize(obj, 'json')
         self.assertEquals(output, expected)
 
     def test_dict(self):
         obj = {}
         expected = '{}'
-        output = Serializer().encode(obj, 'json')
+        output = ObjectSerializer().serialize(obj, 'json')
         self.assertEquals(output, expected)
 
 
@@ -86,17 +86,17 @@ class EncoderTests(SerializationTestCase):
 
     def test_json(self):
         expected = '{"a": 1, "b": "foo", "c": true}'
-        output = Serializer().encode(self.obj, 'json')
+        output = ObjectSerializer().serialize(self.obj, 'json')
         self.assertEquals(output, expected)
 
     def test_yaml(self):
         expected = '{a: 1, b: foo, c: true}\n'
-        output = Serializer().encode(self.obj, 'yaml')
+        output = ObjectSerializer().serialize(self.obj, 'yaml')
         self.assertEquals(output, expected)
 
     def test_xml(self):
         expected = '<?xml version="1.0" encoding="utf-8"?>\n<object><a>1</a><b>foo</b><c>True</c></object>'
-        output = Serializer().encode(self.obj, 'xml')
+        output = ObjectSerializer().serialize(self.obj, 'xml')
         self.assertEquals(output, expected)
 
 
@@ -114,13 +114,13 @@ class BasicSerializerTests(SerializationTestCase):
             'c': True
         }
 
-        self.assertEquals(Serializer().serialize(self.obj), expected)
+        self.assertEquals(ObjectSerializer().serialize(self.obj), expected)
 
     def test_serialize_fields(self):
         """
         Setting 'Meta.fields' specifies exactly which fields to serialize.
         """
-        class CustomSerializer(Serializer):
+        class CustomSerializer(ObjectSerializer):
             class Meta:
                 fields = ('a', 'c')
 
@@ -135,7 +135,7 @@ class BasicSerializerTests(SerializationTestCase):
         """
         Setting 'Meta.exclude' causes a field to be excluded.
         """
-        class CustomSerializer(Serializer):
+        class CustomSerializer(ObjectSerializer):
             class Meta:
                 exclude = ('b',)
 
@@ -150,7 +150,7 @@ class BasicSerializerTests(SerializationTestCase):
         """
         Setting 'Meta.include' causes a field to be included.
         """
-        class CustomSerializer(Serializer):
+        class CustomSerializer(ObjectSerializer):
             class Meta:
                 include = ('_hidden',)
 
@@ -167,7 +167,7 @@ class BasicSerializerTests(SerializationTestCase):
         """
         Both 'Meta.include' and 'Meta.exclude' may be set.
         """
-        class CustomSerializer(Serializer):
+        class CustomSerializer(ObjectSerializer):
             class Meta:
                 include = ('_hidden',)
                 exclude = ('b',)
@@ -184,7 +184,7 @@ class BasicSerializerTests(SerializationTestCase):
         """
         'Meta.fields' overrides both 'Meta.include' and 'Meta.exclude' if set.
         """
-        class CustomSerializer(Serializer):
+        class CustomSerializer(ObjectSerializer):
             class Meta:
                 include = ('_hidden',)
                 exclude = ('b',)
@@ -215,13 +215,13 @@ class SerializeAttributeTests(SerializationTestCase):
             'age': 42
         }
 
-        self.assertEquals(Serializer().serialize(self.obj), expected)
+        self.assertEquals(ObjectSerializer().serialize(self.obj), expected)
 
     def test_serialization_can_include_properties(self):
         """
         Object properties can be included as fields.
         """
-        class CustomSerializer(Serializer):
+        class CustomSerializer(ObjectSerializer):
             class Meta:
                 fields = ('full_name', 'age')
 
@@ -236,7 +236,7 @@ class SerializeAttributeTests(SerializationTestCase):
         """
         Object methods may be included as fields.
         """
-        class CustomSerializer(Serializer):
+        class CustomSerializer(ObjectSerializer):
             class Meta:
                 fields = ('full_name', 'is_child')
 
@@ -261,8 +261,8 @@ class SerializerFieldTests(SerializationTestCase):
         Setting explicit fields on a serializer replaces the default set of
         fields that would have been serialized.
         """
-        class CustomSerializer(Serializer):
-            full_name = Serializer()
+        class CustomSerializer(ObjectSerializer):
+            full_name = ObjectSerializer()
 
         expected = {
             'full_name': 'john doe',
@@ -276,8 +276,8 @@ class SerializerFieldTests(SerializationTestCase):
         have been explicitly included via a Serializer field declaration,
         and regular default object fields will be included.
         """
-        class CustomSerializer(Serializer):
-            full_name = Serializer()
+        class CustomSerializer(ObjectSerializer):
+            full_name = ObjectSerializer()
 
             class Meta:
                 include_default_fields = True
@@ -296,9 +296,9 @@ class SerializerFieldTests(SerializationTestCase):
         A serializer field can take a 'label' argument, which is used as the
         field key instead of the field's property name.
         """
-        class CustomSerializer(Serializer):
-            full_name = Serializer(label='Full name')
-            age = Serializer(label='Age')
+        class CustomSerializer(ObjectSerializer):
+            full_name = ObjectSerializer(label='Full name')
+            age = ObjectSerializer(label='Age')
 
             class Meta:
                 fields = ('full_name', 'age')
@@ -315,9 +315,9 @@ class SerializerFieldTests(SerializationTestCase):
         A serializer field can take a 'source' argument, which is used as the
         field key instead of the field's property name.
         """
-        class CustomSerializer(Serializer):
-            name = Serializer(source='full_name')
-            age = Serializer()
+        class CustomSerializer(ObjectSerializer):
+            name = ObjectSerializer(source='full_name')
+            age = ObjectSerializer()
 
             class Meta:
                 fields = ('name', 'age')
@@ -334,9 +334,9 @@ class SerializerFieldTests(SerializationTestCase):
         Setting source='*', means the complete object will be used when
         serializing that field.
         """
-        class CustomSerializer(Serializer):
-            full_name = Serializer(label='Full name')
-            details = Serializer(fields=('first_name', 'last_name'), label='Details',
+        class CustomSerializer(ObjectSerializer):
+            full_name = ObjectSerializer(label='Full name')
+            details = ObjectSerializer(fields=('first_name', 'last_name'), label='Details',
                                  source='*')
 
             class Meta:
@@ -357,15 +357,15 @@ class SerializerFieldTests(SerializationTestCase):
         A custom serializer can be used with source='*' as serialize the
         complete object within a field.
         """
-        class DetailsSerializer(Serializer):
-            first_name = Serializer(label='First name')
-            last_name = Serializer(label='Last name')
+        class DetailsSerializer(ObjectSerializer):
+            first_name = ObjectSerializer(label='First name')
+            last_name = ObjectSerializer(label='Last name')
 
             class Meta:
                 fields = ('first_name', 'last_name')
 
-        class CustomSerializer(Serializer):
-            full_name = Serializer(label='Full name')
+        class CustomSerializer(ObjectSerializer):
+            full_name = ObjectSerializer(label='Full name')
             details = DetailsSerializer(label='Details', source='*')
 
             class Meta:
@@ -386,10 +386,10 @@ class SerializerFieldTests(SerializationTestCase):
         A serializer field can take a 'serialize' argument, which is used to
         serialize the field value.
         """
-        class CustomSerializer(Serializer):
+        class CustomSerializer(ObjectSerializer):
             full_name = Field(label='Full name',
-                              serialize=lambda name: 'Mr ' + name.title())
-            age = Serializer(label='Age')
+                              convert=lambda name: 'Mr ' + name.title())
+            age = ObjectSerializer(label='Age')
 
             class Meta:
                 fields = ('full_name', 'age')
@@ -417,7 +417,7 @@ class SerializerFieldTests(SerializationTestCase):
         """
         Make sure ordering of serializer fields is preserved.
         """
-        class CustomSerializer(Serializer):
+        class CustomSerializer(ObjectSerializer):
             first_name = Field()
             full_name = Field()
             age = Field()
@@ -470,15 +470,15 @@ class NestedSerializationTests(SerializationTestCase):
             ]
         }
 
-        self.assertEquals(Serializer().serialize(self.obj), expected)
+        self.assertEquals(ObjectSerializer().serialize(self.obj), expected)
 
     def test_nested_serialization_with_args(self):
         """
         We can pass serializer options through to nested fields as usual.
         """
-        class PersonSerializer(Serializer):
+        class PersonSerializer(ObjectSerializer):
             full_name = Field()
-            siblings = Serializer(fields=('full_name',))
+            siblings = ObjectSerializer(fields=('full_name',))
 
         expected = {
             'full_name': 'john doe',
@@ -509,7 +509,7 @@ class NestedSerializationTests(SerializationTestCase):
             ]
         }
 
-        self.assertEquals(Serializer(depth=0).serialize(self.obj), expected)
+        self.assertEquals(ObjectSerializer(depth=0).serialize(self.obj), expected)
 
     def test_depth_one_serialization(self):
         """
@@ -535,7 +535,7 @@ class NestedSerializationTests(SerializationTestCase):
             ]
         }
 
-        self.assertEquals(Serializer(depth=1).serialize(self.obj), expected)
+        self.assertEquals(ObjectSerializer(depth=1).serialize(self.obj), expected)
 
 
 class RecursiveSerializationTests(SerializationTestCase):
@@ -560,7 +560,7 @@ class RecursiveSerializationTests(SerializationTestCase):
                     'father': 'john doe'
             }
         }
-        self.assertEquals(Serializer().serialize(self.obj), expected)
+        self.assertEquals(ObjectSerializer().serialize(self.obj), expected)
 
 
 ##### Simple models without relationships. #####
@@ -583,23 +583,65 @@ class TestSimpleModel(SerializationTestCase):
             finish_time=datetime.datetime(year=2012, month=4, day=30, hour=12, minute=25)
         )
 
-    def test_simple_model(self):
+    def test_simple_dumpdata_json(self):
         self.assertEquals(
-            self.dumpdata.encode(RaceEntry.objects.all(), 'json'),
+            self.dumpdata.serialize(RaceEntry.objects.all(), 'json'),
             serializers.serialize('json', RaceEntry.objects.all())
         )
 
+    def test_simple_dumpdata_yaml(self):
+        self.assertEquals(
+            self.dumpdata.serialize(RaceEntry.objects.all(), 'yaml'),
+            serializers.serialize('yaml', RaceEntry.objects.all())
+        )
+
+    def test_simple_dumpdata_xml(self):
+        self.assertEquals(
+            self.dumpdata.serialize(RaceEntry.objects.all(), 'xml'),
+            serializers.serialize('xml', RaceEntry.objects.all())
+        )
+
     def test_csv(self):
         expected = (
             "id,name,runner_number,start_time,finish_time\r\n"
             "1,John doe,6014,2012-04-30 09:00:00,2012-04-30 12:25:00\r\n"
         )
         self.assertEquals(
-            self.serializer.encode(RaceEntry.objects.all(), 'csv'),
+            self.serializer.serialize(RaceEntry.objects.all(), 'csv'),
             expected
         )
 
 
+class TestNullPKModel(SerializationTestCase):
+    def setUp(self):
+        self.dumpdata = DumpDataSerializer()
+        self.serializer = ModelSerializer(depth=0)
+        self.objs = [RaceEntry(
+            name='John doe',
+            runner_number=6014,
+            start_time=datetime.datetime(year=2012, month=4, day=30, hour=9),
+            finish_time=datetime.datetime(year=2012, month=4, day=30, hour=12, minute=25)
+        )]
+
+    def test_null_pk_dumpdata_json(self):
+        self.assertEquals(
+            self.dumpdata.serialize(self.objs, 'json'),
+            serializers.serialize('json', self.objs)
+        )
+
+    def test_null_pk_dumpdata_yaml(self):
+        self.assertEquals(
+            self.dumpdata.serialize(self.objs, 'yaml'),
+            serializers.serialize('yaml', self.objs)
+        )
+
+    def test_null_pk_dumpdata_xml(self):
+        self.assertEquals(
+            self.dumpdata.serialize(self.objs, 'xml'),
+            serializers.serialize('xml', self.objs)
+        )
+
+
 ##### Model Inheritance #####
 
 class Account(models.Model):
@@ -623,7 +665,7 @@ class TestModelInheritance(SerializationTestCase):
 
     def test_dumpdata_child_model(self):
         self.assertEquals(
-            self.dumpdata.encode(PremiumAccount.objects.all(), 'json'),
+            self.dumpdata.serialize(PremiumAccount.objects.all(), 'json'),
             serializers.serialize('json', PremiumAccount.objects.all())
         )
 
@@ -682,16 +724,36 @@ class TestNaturalKey(SerializationTestCase):
             name='frogger'
         )
 
-    def test_naturalkey_dumpdata(self):
+    def test_naturalkey_dumpdata_json(self):
         """
         Ensure that we can replicate the existing dumpdata
         'use_natural_keys' behaviour.
         """
         self.assertEquals(
-            self.dumpdata.encode(Pet.objects.all(), 'json', use_natural_keys=True),
+            self.dumpdata.serialize(Pet.objects.all(), 'json', use_natural_keys=True),
             serializers.serialize('json', Pet.objects.all(), use_natural_keys=True)
         )
 
+    def test_naturalkey_dumpdata_yaml(self):
+        """
+        Ensure that we can replicate the existing dumpdata
+        'use_natural_keys' behaviour.
+        """
+        self.assertEquals(
+            self.dumpdata.serialize(Pet.objects.all(), 'yaml', use_natural_keys=True),
+            serializers.serialize('yaml', Pet.objects.all(), use_natural_keys=True)
+        )
+
+    def test_naturalkey_dumpdata_xml(self):
+        """
+        Ensure that we can replicate the existing dumpdata
+        'use_natural_keys' behaviour.
+        """
+        self.assertEquals(
+            self.dumpdata.serialize(Pet.objects.all(), 'xml', use_natural_keys=True),
+            serializers.serialize('xml', Pet.objects.all(), use_natural_keys=True)
+        )
+
     def test_naturalkey(self):
         """
         Ensure that we can use NaturalKeyRelatedField to represent foreign
@@ -767,19 +829,19 @@ class TestOneToOneModel(SerializationTestCase):
 
     def test_onetoone_dumpdata_json(self):
         self.assertEquals(
-            self.dumpdata.encode(Profile.objects.all(), 'json'),
+            self.dumpdata.serialize(Profile.objects.all(), 'json'),
             serializers.serialize('json', Profile.objects.all())
         )
 
     def test_onetoone_dumpdata_yaml(self):
         self.assertEquals(
-            self.dumpdata.encode(Profile.objects.all(), 'yaml'),
+            self.dumpdata.serialize(Profile.objects.all(), 'yaml'),
             serializers.serialize('yaml', Profile.objects.all())
         )
 
     def test_onetoone_dumpdata_xml(self):
         self.assertEquals(
-            self.dumpdata.encode(Profile.objects.all(), 'xml'),
+            self.dumpdata.serialize(Profile.objects.all(), 'xml'),
             serializers.serialize('xml', Profile.objects.all())
         )
 
@@ -892,19 +954,19 @@ class TestFKModel(SerializationTestCase):
 
     def test_fk_dumpdata_json(self):
         self.assertEquals(
-            self.dumpdata.encode(Vehicle.objects.all(), 'json'),
+            self.dumpdata.serialize(Vehicle.objects.all(), 'json'),
             serializers.serialize('json', Vehicle.objects.all())
         )
 
     def test_fk_dumpdata_yaml(self):
         self.assertEquals(
-            self.dumpdata.encode(Vehicle.objects.all(), 'yaml'),
+            self.dumpdata.serialize(Vehicle.objects.all(), 'yaml'),
             serializers.serialize('yaml', Vehicle.objects.all())
         )
 
     def test_fk_dumpdata_xml(self):
         self.assertEquals(
-            self.dumpdata.encode(Vehicle.objects.all(), 'xml'),
+            self.dumpdata.serialize(Vehicle.objects.all(), 'xml'),
             serializers.serialize('xml', Vehicle.objects.all())
         )
 
@@ -1012,21 +1074,21 @@ class TestManyToManyModel(SerializationTestCase):
 
     def test_m2m_dumpdata_json(self):
         self.assertEquals(
-            self.dumpdata.encode(Book.objects.all(), 'json'),
+            self.dumpdata.serialize(Book.objects.all(), 'json'),
             serializers.serialize('json', Book.objects.all())
         )
         self.assertEquals(
-            self.dumpdata.encode(Author.objects.all(), 'json'),
+            self.dumpdata.serialize(Author.objects.all(), 'json'),
             serializers.serialize('json', Author.objects.all())
         )
 
     def test_m2m_dumpdata_yaml(self):
         self.assertEquals(
-            self.dumpdata.encode(Book.objects.all(), 'yaml'),
+            self.dumpdata.serialize(Book.objects.all(), 'yaml'),
             serializers.serialize('yaml', Book.objects.all())
         )
         self.assertEquals(
-            self.dumpdata.encode(Author.objects.all(), 'yaml'),
+            self.dumpdata.serialize(Author.objects.all(), 'yaml'),
             serializers.serialize('yaml', Author.objects.all())
         )
 
@@ -1035,11 +1097,11 @@ class TestManyToManyModel(SerializationTestCase):
         # dumpdata = DumpDataSerializer()
         # dumpdata.fields['fields'].opts.preserve_field_order = True
         self.assertEquals(
-            self.dumpdata.encode(Book.objects.all(), 'xml'),
+            self.dumpdata.serialize(Book.objects.all(), 'xml'),
             serializers.serialize('xml', Book.objects.all())
         )
         self.assertEquals(
-            self.dumpdata.encode(Author.objects.all(), 'xml'),
+            self.dumpdata.serialize(Author.objects.all(), 'xml'),
             serializers.serialize('xml', Author.objects.all())
         )
 
@@ -1069,3 +1131,42 @@ class TestManyToManyModel(SerializationTestCase):
             self.flat_model.serialize(Book.objects.get(id=1)),
             expected
         )
+
+
+class Anchor(models.Model):
+    data = models.CharField(max_length=30)
+
+    class Meta:
+        ordering = ('id',)
+
+
+class M2MIntermediateData(models.Model):
+    data = models.ManyToManyField(Anchor, null=True, through='Intermediate')
+
+
+class Intermediate(models.Model):
+    left = models.ForeignKey(M2MIntermediateData)
+    right = models.ForeignKey(Anchor)
+    extra = models.CharField(max_length=30, blank=True, default="doesn't matter")
+
+
+class TestManyToManyThroughModel(SerializationTestCase):
+    """
+    Test one-to-one field relationship on a model with a 'through' relationship.
+    """
+    def setUp(self):
+        self.dumpdata = DumpDataSerializer()
+        right = Anchor.objects.create(data='foobar')
+        left = M2MIntermediateData.objects.create()
+        Intermediate.objects.create(extra='wibble', left=left, right=right)
+        self.obj = left
+
+    def test_m2m_through_dumpdata_json(self):
+        self.assertEquals(
+            self.dumpdata.serialize(M2MIntermediateData.objects.all(), 'json'),
+            serializers.serialize('json', M2MIntermediateData.objects.all())
+        )
+        self.assertEquals(
+            self.dumpdata.serialize(Anchor.objects.all(), 'json'),
+            serializers.serialize('json', Anchor.objects.all())
+        )
